#!/usr/bin/perl
#
# rpm.cgi - generates HTML pages with info on installed RPMs
# Copyright (C) 1999 Abdul-Wahid Paterson (eesa@webstar.co.uk)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#############################################################################


use config;
use text_conv;




# Assign data items returned from string
&parse_form_data(*form);
$rpm = $form{'rpm'};
$op = $form{'op'};
$withwhat = $form{'withwhat'};
$cache = $form{'cache'};
$page = $form{'page'};
$image = $form{'image'};

# Check to see which operation we should be doing
if($image) {
    &image_print($image);
}
elsif($op eq "change") {
    &change_log($rpm);
}
elsif($op eq "build") {
    &build_detail($rpm);
}
elsif($op eq "files") {
    &files_info($rpm);
}
elsif($op eq "deps") {
    &dependents($rpm);
}
elsif($op eq "help") {
    &generic_help($rpm, $withwhat);
}
elsif($rpm) {
    &rpm_parts($rpm);
}
else {
    &rpms_main();
}

exit(0);


#############################################################################


# Parse the data given to the script

sub parse_form_data {
    local (*FORM_DATA) = @_;
    local ($request_method, $query_string, @key_value_pairs, $key_value, $key, $value);

    $request_method = $ENV{'REQUEST_METHOD'};
    if ($request_method eq "GET") {
	$query_string =$ENV{'QUERY_STRING'};
    }
    elsif ($request_method eq "POST") {
	read (STDIN, $query_string, $ENV{'CONTENT_LENGTH'});
    }
    else {
	&return_error (500, "Server Error", "Unsupported method");
    }
    
    @key_value_pairs = split (/&/, $query_string);
    
    foreach $key_value (@key_value_pairs) {
	($key, $value) = split (/=/, $key_value);
	$value =~ tr/+/ /;
	$value =~ s/%([dA-Fa-f][dA-Fa-f])/pack ("C", hex ($1))/eg;
	$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

	if (defined($FORM_DATA{$key})) {
	    $FORM_DATA{$key} = join ("\0", $FORM_DATA{$key}, $value);
	}
	else {
	    $FORM_DATA{$key} = $value;
	}
    }
}


sub return_error {
    local ($status, $keyword, $message) = @_;

    print "Content-type: text/html", "\n";
    print "Status: ", $status, " ", $keyword, "\n\n";

    print <<End_of_Error;
<TITLE> "CGI Program - Unexpected Error</TITLE>
<H1>$keyword</H1>
<HR>$message<HR>
Please contact webmaster@$hostname for more information.

End_of_Error

        exit(1);
}




#############################################################################

# This function returns the main web page listing all RPMs

sub rpms_main {

    # Set up some stuff
    $rpm_count = 0;
    $date = `$date_prog`;
    chomp($date);

    $server = `$hostname_prog`;
    chomp($server);

    if(($cache) && (-r "$temp_dir/.rpm_cgi.$cache") && ($use_cache_file == 1)) {
	# we have a previous cached file to read
	
	$table_count = 0;

	print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
        <TITLE>RPM Information for $server</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<BR>
<CENTER>
<A HREF="http://eesa.webstar.co.uk/rpm_cgi/">
<IMG SRC="rpm.cgi?image=rpm_cgi.gif" ALT="RPM.CGI" BORDER=0>
</A>
<BR><BR>
<H1>RPM Information for $server</H1>
<BR>
<H2>on $date</H2>
<BR><BR>
<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000>
<TR><TD>
<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>
<TR>
   <TD WIDTH=50%><B>RPM</B></TD>
   <TD WIDTH=25%><B>Version</B></TD>
   <TD WIDTH=25%><B>Release</B></TD>
<TR>
END_HTML

 
    open(CACHE, "$temp_dir/.rpm_cgi.$cache");
	
	while($line = <CACHE>) {
	    ($rpm, $version, $release) = split /%/, $line;
	    $table_count++;
	    if($table_count == $table_length) {
		print "</TABLE></TD></TR></TABLE>\n";
		print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
		print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
		$table_count = 0;
	    }
	    print "<TR>\n   <TD WIDTH=50%><A HREF=\"rpm.cgi?rpm=$rpm&cache=$cache\">$rpm</A></TD>\n";
	    print "   <TD WIDTH=25%>$version</TD>\n";
	    print "   <TD WIDTH=25%>$release</TD>\n</TR>\n";
	    $rpm_count++;
	}
	
	close(CACHE);
     


    }
    else {
	
	# Get rid of any unwanted cache files

	$useless = `rm -f $temp_dir/.rpm_cgi*`;
	
	#Get a list of RPMS
	$rpms = `$rpm_prog -qa --queryformat %{NAME}"\n"`;

	
	# Sort the rpms
	
	$sort_rpm = "null";
	$rpm_count = 0;

	while($sort_rpm) {
	    ($sort_rpm, $rpms) = split /\n/, $rpms, 2;
	    if($sort_rpm) {
		print TEMP "$sort_rpm\n";
		@the_rpms[$rpm_count] = $sort_rpm;
		$rpm_count++;
	    }
	}
	@the_rpms = sort @the_rpms; 


	# get epoc to mark temp cache file
	$epoc = `$date_prog +%s`;
	chomp($epoc);
	$cache = "$temp_dir/.rpm_cgi.$epoc";
	
	# Output the webpage

	$table_count = 0;

	print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
        <TITLE>RPM Information for $server</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<BR>
<CENTER>
<A HREF="http://eesa.webstar.co.uk/rpm_cgi/">
<IMG SRC="rpm.cgi?image=rpm_cgi.gif" ALT="RPM.CGI" BORDER=0>
</A>
<BR><BR>
<H1>RPM Information for $server</H1>
<BR>
<H2>on $date</H2>
<BR><BR>
<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000>
<TR><TD>
<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>
<TR>
   <TD WIDTH=50%><B>RPM</B></TD>
   <TD WIDTH=25%><B>Version</B></TD>
   <TD WIDTH=25%><B>Release</B></TD>
<TR>
END_HTML

    $table_count++;
    if($table_count == $table_length) {
	print "</TABLE></TD></TR></TABLE>\n";
	print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
	print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
	$table_count = 0;
    }


	open(CACHE, ">$cache");
	foreach $rpm (@the_rpms) {
	    print "<TR>\n   <TD WIDTH=50%><A HREF=\"rpm.cgi?rpm=$rpm&cache=$epoc\">$rpm</A></TD>\n";
	    print CACHE "$rpm%";
	    $version = `$rpm_prog -q $rpm --queryformat %{VERSION}`;
	    print "   <TD WIDTH=25%>$version</TD>\n";
	    print CACHE "$version%";
	    $release = `$rpm_prog -q $rpm --queryformat %{RELEASE}`;
	    print "   <TD WIDTH=25%>$release</TD>\n</TR>\n";
	    print CACHE "$release\n";
	    $table_count++;
	    if($table_count == $table_length) {
		print "</TABLE></TD></TR></TABLE>\n";
		print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
		print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
		$table_count = 0;
	    }
	}

	close(CACHE);
    }
	print <<END_HTML;
<TR>
   <TD><B>TOTAL</B></TD>
   <TD COLSPAN=2 ALIGN="CENTER"><B>$rpm_count</B>
</TABLE>
</TD></TR></TABLE>
<BR><BR>
<TABLE WIDTH=85% CELLPADDING=20 BORDER=0>
<TR>
   <TD ALIGN=CENTER>
      <A HREF="http://www.linux.org/"><IMG SRC="rpm.cgi?image=linux.jpg" ALT="Linux Site" BORDER=0></A>
   </TD>
<TR>
</TR>
   <TD ALIGN=CENTER>
      <A HREF="http://www.rpm.org/"><IMG SRC="rpm.cgi?image=rpm.gif" ALT="RPM Site" BORDER=0></A>
   </TD>
</TR>
</TABLE>
</CENTER>
<BR><BR>
</BODY>
</HTML>
END_HTML
}



#############################################################################

# This function looks at the basic information about a particular RPM

sub rpm_parts {
    local ($rpm) = @_;

    print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
    <TITLE>Package Information for $rpm</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<BR>
<CENTER>
<H1>Package Information for $rpm</H1>
<BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Package%20Information">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000>
<TR><TD>
<TABLE WIDTH=100% BORDER=0 CELLPADDING=10 BGCOLOR=#FFFFBB>
END_HTML

    $summary = `$rpm_prog -q $rpm --queryformat %{SUMMARY}`;
    print "<TR><TD VALIGN=\"top\"><B>Summary</B></TD><TD>$summary</TD></TR>\n";
    $group = `$rpm_prog -q $rpm --queryformat %{GROUP}`;
    print "<TR><TD VALIGN=\"top\"><B>Group</B></TD><TD>$group</TD></TR>\n";
    $installtime = `$rpm_prog -q $rpm --queryformat %{INSTALLTIME}`;
    print "<TR><TD VALIGN=\"top\"><B>Install Time</B></TD><TD>";
    &print_date($installtime);
    print "</TD></TR>\n";
    $rpm_size = `$rpm_prog -q $rpm --queryformat %{SIZE}`;
    print "<TR><TD VALIGN=\"top\"><B>Size</B></TD><TD>$rpm_size Bytes</TD></TR>\n";
    $copyright = `$rpm_prog -q $rpm --queryformat %{COPYRIGHT}`;
    print "<TR><TD VALIGN=\"top\"><B>Copyright</B></TD><TD>$copyright</TD></TR>\n";
    $description = `$rpm_prog -q $rpm --queryformat %{DESCRIPTION}`;
    $description = &text_conv($description);
    print "<TR><TD VALIGN=\"top\"><B>Description</B></TD><TD>$description</TD></TR>\n";

print <<END_HTML;
</TABLE>
</TD></TR></TABLE>
<BR><BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&"cache=$cache>
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&"cache=$cache>
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Package%20Information">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
</CENTER>
</BODY>
</HTML>
END_HTML
}

#############################################################################

# This function prints the change log for an RPM

sub change_log {

    local ($rpm) = @_;

    print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
    <TITLE>Change Log for $rpm</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<BR>
<CENTER>
<H1>Change Log for $rpm</H1>
<BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Change%20Log">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000>
<TR><TD>
<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>
<TR>
<TD>
END_HTML

    $change = `$rpm_prog -q --changelog $rpm`;
    $change = &text_conv($change);
    print $change;

print <<END_HTML;
</TD>
</TR>
</TABLE>
</TD></TR></TABLE>
<BR><BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Change%20Log">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
</CENTER>
</BODY>
</HTML>
END_HTML
}


#############################################################################

# This function prints the build information for an RPM

sub build_detail {

    local ($rpm) = @_;

    print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
    <TITLE>Build Details for $rpm</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<CENTER>
<BR>
<H1>Build Details for $rpm</H1>
<BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Build%20Detail">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000>
<TR><TD>
<TABLE WIDTH=100% BORDER=0 CELLPADDING=10 BGCOLOR=#FFFFBB>
END_HTML

    $distribution = `$rpm_prog -q $rpm --queryformat %{DISTRIBUTION}`;
    print "<TR><TD VALIGN=\"top\"><B>Distribution</B></TD><TD>$distribution</TD></TR>\n";
    $buildhost = `$rpm_prog -q $rpm --queryformat %{BUILDHOST}`; 
    print "<TR><TD VALIGN=\"top\"><B>Build Host</B></TD><TD>$buildhost</TD></TR>\n";
    $buildtime = `$rpm_prog -q $rpm --queryformat %{BUILDTIME}`;
    print "<TR><TD VALIGN=\"top\"><B>Build Time</B></TD><TD>";
    &print_date($buildtime);
    print "</TD></TR>\n";
    $url = `$rpm_prog -q $rpm --queryformat %{URL}`;
    if($url eq "(none)") {
	print "<TR><TD VALIGN=\"top\"><B>URL</B></TD><TD>$url</TD></TR>\n";
    }
    else {
	print "<TR><TD VALIGN=\"top\"><B>URL</B></TD><TD>";
	print "<A HREF=\"$url\">$url</A></TD></TR>\n";
    }
    $packager = `$rpm_prog -q $rpm --queryformat %{PACKAGER}`;
    print "<TR><TD VALIGN=\"top\"><B>Packager</B></TD><TD>$packager</TD></TR>\n";
    $vendor = `$rpm_prog -q $rpm --queryformat %{VENDOR}`;
    print "<TR><TD VALIGN=\"top\"><B>Vendor</B></TD><TD>$vendor</TD></TR>\n";
    $srpm = `$rpm_prog -q $rpm --queryformat %{SOURCERPM}`;
    print "<TR><TD VALIGN=\"top\"><B>Source RPM</B></TD><TD>$srpm</TD></TR>\n";
    $rpm_version = `$rpm_prog -q $rpm --queryformat %{RPMVERSION}`;
    print "<TR><TD VALIGN=\"top\"><B>RPM Version</B></TD><TD>$rpm_version</TD></TR>\n";

    print <<END_HTML;
</TABLE>
</TD></TR></TABLE>
<BR><BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Build%20Detail">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
</CENTER>
</BODY>
</HTML>
END_HTML
}

#############################################################################

# This function prints the file information for an RPM

sub files_info {

    local ($rpm) = @_;
    
    $table_count = 0;
    if(! $page) {
	$page = 0;
    }
    $page_count = 0;

    print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
    <TITLE>File Information for $rpm</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<BR>
<CENTER>
<H1>File Information for $rpm</H1>
<BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=File%20Checking">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
END_HTML



    # get a list of all the files in the RPM
    $files = `$rpm_prog -ql $rpm`;
    
    # get a list of the conf and doc files
    $conf_files = `$rpm_prog -qc $rpm`;
    $doc_files = `$rpm_prog -qd $rpm`;
    

    # get a list of all the "other"
    $conf_files_copy = $conf_files;
    $doc_files_copy = $doc_files;
    $files_copy = $files;
    $other_files = "";

    # count how many files in each
    $conf_count = 0;
    $doc_count = 0;
    $other_count = 0;

    $test_file = "null";
    while($test_file) {
	($test_file, $files_copy) = split /\n/, $files_copy, 2;
	if($test_file) {
	    # we are testing for a particular file
	    $not_this_one = 0;
	    # first lets check against all documents
	    $test_doc = "null";
	    while(($test_doc) && ($not_this_one == 0)) {
		($test_doc, $doc_files_copy) = split /\n/, $doc_files_copy, 2;
		if($test_doc) {
		    if($test_doc eq $test_file) {
			$not_this_one = 1;
			$doc_count++;
		    }
		}
	    }
	    $test_conf = "null";
	    while(($test_conf) && ($not_this_one == 0)) {
		($test_conf, $conf_files_copy) = split /\n/, $conf_files_copy, 2;
		if($test_conf) {
		    if($test_conf eq $test_file) {
			$not_this_one = 1;
			$conf_count++;
		    }
		}
	    }
	    if($not_this_one == 0) {
		# We haven't matched this one so it an "other";
		$other_files = $other_files . "$test_file\n";
		$other_count++;
	    }
	}
	$conf_files_copy = $conf_files;
	$doc_files_copy = $doc_files;
    }
    $total_count = $doc_count + $conf_count + $other_count;


    # cut the page according to the page length unless 0
    $real_doc_files = "";
    $real_conf_files = "";
    $real_other_files = "";

    if($page_length != 0) {
	# start counting how many we have
	$this_count = 0;
		    
	$test_conf = "null";
	while(($test_conf) && ($this_count < ($page_length + $page)))  {
	    ($test_conf, $conf_files) = split /\n/, $conf_files, 2;
	    if($test_conf) {
	       if($this_count < $page) {
		    # we haven't got there yet we need to chop
		    $this_count++;
		}
		elsif(($this_count >= $page) && ($this_count < ($page_length + $page))) {
		    # we are in the page bit that we want
		    $real_conf_files = $real_conf_files . "$test_conf\n";
		    $this_count ++;
		}
		else {
		    # we have gone past the page bit we want
		    $this_count++;
		}
	    }
	}
	$test_doc = "null";
	while(($test_doc)  && ($this_count < ($page_length + $page))) {
	    ($test_doc, $doc_files) = split /\n/, $doc_files, 2;
	    if($test_doc) {
		if($this_count < $page) {
		    # we haven't got there yet we need to chop
		    $this_count++;
		}
		elsif(($this_count >= $page) && ($this_count < ($page_length + $page))) {
		    # we are in the page bit that we want
		    $real_doc_files = $real_doc_files . "$test_doc\n";
		    $this_count ++;
		}
		else {
		    # we have gone past the page bit we want
		    $this_count++;
		}
	    }
	}
	$test_other = "null";
	while(($test_other) && ($this_count < ($page_length + $page)))  {
	    ($test_other, $other_files) = split /\n/, $other_files, 2;
	    if($test_other) {
	       if($this_count < $page) {
		    # we haven't got there yet we need to chop
		    $this_count++;
		}
		elsif(($this_count >= $page) && ($this_count < ($page_length + $page))) {
		    # we are in the page bit that we want
		    $real_other_files = $real_other_files . "$test_other\n";
		    $this_count ++;
		}
		else {
		    # we have gone past the page bit we want
		    $this_count++;
		}
	    }
	}
    }
    $doc_files = $real_doc_files;
    $conf_files = $real_conf_files;
    $other_files = $real_other_files;


    # For each section, documentation, configuration and other files

    for($i = 0; $i < 3; $i++) {
	$these_files = "";
	if(($i == 0) && ($conf_files)) {
	    # Configuration first
	    print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>\n";
	    print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>\n";
	    print "<TR><TD COLSPAN=9><H1>Configuration Files</H1></TD></TR>\n";
	    $these_files = $conf_files;
	}
	elsif(($i == 1) && ($doc_files)) {
	    # Documentation files second
	    print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>\n";
	    print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>\n";
	    print "<TR><TD COLSPAN=9><H1>Documentation Files</H1></TD></TR>\n";
	    $these_files = $doc_files;
	}
	elsif(($i == 2) && ($other_files)) {
	    # "Other" files
	    print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>\n";
	    print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>\n";
	    print "<TR><TD COLSPAN=9><H1>Other Files</H1></TD></TR>\n";
	    $these_files = $other_files;
	}
	if($these_files) {
	    $table_count++;
	    if($table_count == $table_length) {
		print "</TABLE></TD></TR></TABLE>\n";
		print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
		print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
		$table_count = 0;
	    }
	    print "<TR><TD WIDTH=95%><B>File</></TD><TD><B>S</B></TD><TD><B>M</B></TD><TD><B>5</B></TD><TD><B>D</B></TD>";
	    print "<TD><B>L</B></TD><TD><B>U</B></TD><TD><B>G</B></TD><TD><B>T</B></TD><TR>\n";
	    
	    
	    $table_count++;
	    if($table_count == $table_length) {
		print "</TABLE></TD></TR></TABLE>\n";
		print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
		print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
		$table_count = 0;
	    }


	    # get a list of all the file changes for an RPM
	    $checksums_orig = `$rpm_prog -V $rpm`;
	    chomp($checksums_orig);
	    
	    $file = "null";
	    # for every file check to see if it has changed
	    while($file) {
		($file, $these_files) = split /\n/, $these_files, 2;
		if($file) {
		    $checksums = $checksums_orig;
		    $check_file = "null";
		    $printed = 0;
		    while($check_file) {
			# take the first file to be checked for
			($check_file, $checksums) = split /\n/, $checksums, 2;
			if($check_file) {
			    
			    # the file name starts from the 11 character
			    $this_file = substr $check_file, 11;
			    if($this_file eq $file) {
			    
			    # "missing" is a special case, there may be others too, please let me know
				$front = substr $check_file, 0, 7;
				if($front eq "missing") {
				    print "<TR>\n<TD WIDTH=95%><FONT COLOR=\"red\">$file</FONT></TD>\n";
				    print "<TD COLSPAN=8 ALIGN=CENTER>";
				    print "<FONT COLOR=\"red\" SIZE=+1>MISSING";
				    print "</FONT></TD>\n</TR>\n";
				
				    $table_count++;
				    if($table_count == $table_length) {
					print "</TABLE></TD></TR></TABLE>\n";
					print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
					print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
					$table_count = 0;
				    }
				}
				else {
				    
				    # split the file modification details and print them
				    ($chks, $chkm, $chk5, $chkd, $chkl, $chku, $chkg, $chkt) = split //, $check_file;
				    print "<TR>\n<TD WIDTH=95%><FONT COLOR=\"red\">$file</FONT></TD>\n";
				    print "<TD ALIGN=CENTER>";
				    if($chks eq "S") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TD ALIGN=CENTER>";
				    if($chkm eq "M") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TD ALIGN=CENTER>";
				    if($chk5 eq "5") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TD ALIGN=CENTER>";
				    if($chkd eq "D") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TD ALIGN=CENTER>";
				    if($chkl eq "L") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TD ALIGN=CENTER>";
				    if($chku eq "U") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TD ALIGN=CENTER>";
				    if($chkg eq "G") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TD ALIGN=CENTER>";
				    if($chkt eq "T") {
					print "<IMG SRC=\"rpm.cgi?image=cross.gif\" ALT=\"Red\" BORDER=0></TD>\n";
				    }
				    else {
					print "<IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
				    }
				    print "<TR>\n";
				    $table_count++;
				    if($table_count == $table_length) {
					print "</TABLE></TD></TR></TABLE>\n";
					print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
					print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
					$table_count = 0;
				    }
				    $printed = 1;
				}
			    }
			}
		    }
		    if($printed == 0) {
			
			# the file wasn't modified so we can return the Green Tick
			print "<TR>\n<TD WIDTH=95%>$file</TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			print "<TD ALIGN=CENTER><IMG SRC=\"rpm.cgi?image=tick.gif\" ALT=\"Green\" BORDER=0></TD>\n";
			
			$table_count++;
			if($table_count == $table_length) {
			    print "</TABLE></TD></TR></TABLE>\n";
			    print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
			    print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
			    $table_count = 0;
			}
		    }
		}
	    }
	    print "</TABLE></TD></TR></TABLE><BR><BR>\n";
	}
    }

    if(($total_count > ($page + $page_length)) && ($page == 0))  {
	$new_page = $page + $page_length;
	print "<BR><BR><TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
	print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
	print "<TR><TD WIDTH=50% ALIGN=CENTER>Previous $page_length</TD>";
	print "<TD WIDTH=50% ALIGN=CENTER>";
	print "<A HREF=\"rpm.cgi?rpm=$rpm&cache=$cache&op=files&page=$new_page\">Next $page_length</A>";
	print "</TD></TR></TABLE></TD></TR></TABLE><BR><BR>\n";
    }
    elsif(($total_count > ($page + $page_length)) && ($page != 0))  {
	$new_page = $page + $page_length;
	$prev_page = $page - $page_length;
	print "<BR><BR><TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
	print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
	print "<TR><TD WIDTH=50% ALIGN=CENTER>";
	print "<A HREF=\"rpm.cgi?rpm=$rpm&cache=$cache&op=files&page=$prev_page\">Previous $page_length</A>";
	print "</TD><TD WIDTH=50% ALIGN=CENTER>";
	print "<A HREF=\"rpm.cgi?rpm=$rpm&cache=$cache&op=files&page=$new_page\">Next $page_length</A>";
	print "</TD></TR></TABLE></TD></TR></TABLE><BR><BR>\n";
    }
    elsif($page != 0) {
	$prev_page = $page - $page_length;
	print "<BR><BR><TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
	print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
	print "<TR><TD WIDTH=50% ALIGN=CENTER>";
	print "<A HREF=\"rpm.cgi?rpm=$rpm&cache=$cache&op=files&page=$prev_page\">Previous $page_length</A>";
	print "</TD><TD WIDTH=50% ALIGN=CENTER>";
	print "Next $page_length";
	print "</TD></TR></TABLE></TD></TR></TABLE><BR><BR>\n";
    }


    print <<END_HTML;

<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=File%20Checking">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
</CENTER>
</BODY>
</HTML>
END_HTML
}


#############################################################################

# this function returns the depencency information for an RPM

sub dependents {
    local ($rpm) = @_;

    print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
    <TITLE>Dependency Information for $rpm</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<BR>
<CENTER>
<H1>Dependency Information for $rpm</H1>
<BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Dependency%20Information">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000>
<TR><TD>
<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>
END_HTML

    print "<TR>\n<TD VALIGN=top WIDTH=25%><B>Provides</B></TD>\n";
    $provides = `$rpm_prog -q --provides $rpm`;
    print "<TD><PRE>$provides</PRE></TD>\n</TR>";
    print "</TABLE>";
    print "</TD></TR></TABLE><BR>";

    print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
    print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";

    print "<TR>\n<TD VALIGN=top WIDTH=25%><B>Requires</B></TD>\n";
    print "<TD>";


    $requires = `$rpm_prog -q --requires $rpm`;
    $needy = "null";
    
    if($requires eq "(none)") {
	# then there are no requirements for this package
	print "<PRE>$requires</PRE>";
    }
    else {
	print "<TABLE WIDTH=100% CELLPADDING=4>\n";
	while($needy) {
	    ($needy, $requires) = split /\n/, $requires, 2;
	    # we don't care about version numbers so strip everything after first blank
	    ($realneedy, $rest) = split / /, $needy;
	    if($realneedy) {
		# check if this need is supplied by any RPM
		$the_provider = `$rpm_prog -q --whatprovides $realneedy --queryformat %{NAME}`;
		$possible_rpm = `$rpm_prog -q $realneedy --queryformat %{NAME}`;
		if($the_provider) {

		    # we found a match for the requirement
		    print "<TR><TD>$needy</TD>";
		    print "<TD><A HREF=\"rpm.cgi?rpm=$the_provider&cache=$cache\">";
		    print "$the_provider</A></TD></TR>";
		    $the_provider = "";
		}
		elsif($possible_rpm eq $realneedy) {
		    print "<TR><TD>$needy</TD>";
		    print "<TD><A HREF=\"rpm.cgi?rpm=$possible_rpm&cache=$cache\">";
		    print "$possible_rpm</A></TD></TR>";
		}
		else {
		    
		    # we may still be able to link it to a file
		    if(-e $realneedy) {
			#it is a file on the system
			$the_provider = `$rpm_prog -qf $realneedy --queryformat %{NAME}`;
			if($the_provider) {
			    print "<TR><TD>$needy</TD>";
			    print "<TD><A HREF=\"rpm.cgi?rpm=$the_provider&cache=$cache\">";
			    print "$the_provider</A></TD></TR>";
			    $the_provider = "";
			}
			else {
			    print "<TR><TD>$needy</TD><TD> no provider was found</TD></TR>";
			}
		    }
		    else {
			# it could still be a file on the system if it is not expressed as a full pathname
			# lets check to see if it could be one of these paths

			@possible_paths = qw{ /bin /sbin /usr/bin /usr/sbin /usr/local/bin /usr/local/sbin /usr/X11R6/bin };
			$found_it = 0;

			foreach $possible_path (@possible_paths) {

			    if((-e "$possible_path/$realneedy") && ($found_it == 0)) {

				# we have found it on the system
				$the_provider = `$rpm_prog -qf $possible_path/$realneedy --queryformat %{NAME}`;
				if($the_provider) {
				    print "<TR><TD>$needy</TD>";
				    print "<TD><A HREF=\"rpm.cgi?rpm=$the_provider&cache=$cache\">";
				    print "$the_provider</A></TD></TR>";
				    $the_provider = "";
				    $found_it = 1;
				}
			    }
			}
			if($found_it == 0) {
			    # we still didn't find it - I wonder where it is
			    print "<TR><TD>$needy</TD><TD> no provider was found</TD></TR>";
			}
		    }
		}
	    }
	}
	print "</TABLE>";
    }
			
    print "</TD>\n</TR>";
    print "</TABLE>";
    print "</TD></TR></TABLE><BR>";

    print "<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>";
    print "<TABLE BORDER=0 WIDTH=100% CELLPADDING=10 BGCOLOR=#FFFFBB>";
    print "<TR>\n<TD VALIGN=top WIDTH=25%><B>Obsoletes</B></TD>\n";
    $obsoletes = `$rpm_prog -q $rpm --queryformat %{OBSOLETES}`;
    print "<TD><PRE>$obsoletes</PRE></TD>\n</TR>";

    print <<END_HTML;
</TABLE>
</TD></TR></TABLE>
<BR><BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=help&cache=$cache&withwhat=Dependency%20Information">
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0></A>
<BR><BR>
</CENTER>
</BODY>
</HTML>
END_HTML
}


#############################################################################

# The function prints out the template for a help page

sub generic_help {

    local ($rpm, $withwhat) = @_;

    print <<END_HTML;
Content-type: text/html

<HTML>
<HEAD>
    <TITLE>Help for $withwhat</TITLE>
</HEAD>
<BODY BGCOLOR=#FFFFFF>
<BR>
<CENTER>
<H1>Help for $withwhat</H1>
<BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0>
<BR><BR>
<TABLE BORDER=0 WIDTH=85% CELLPADDING=2 BGCOLOR=#000000><TR><TD>
<TABLE BORDER=0 WIDTH=100% CELLPADDING=20 BGCOLOR=#FFFFBB>
<TR><TD>
<CENTER><H2>Help for $withwhat</H2></CENTER>
END_HTML

    if($withwhat eq "Dependency Information") {
	&help_dependents();
    }
    elsif($withwhat eq "File Checking") {
	&help_file_checking();
    }
    elsif($withwhat eq "Build Detail") {
	&help_build_detail();
    }
    elsif($withwhat eq "Package Information") {
	&help_package_information();
    }
    elsif($withwhat eq "Change Log") {
	&help_change_log();
    }

    print <<END_HTML;
</TD></TR>
</TABLE>
</TD></TR></TABLE>
<BR><BR>
<A HREF="rpm.cgi?cache=$cache">
<IMG SRC="rpm.cgi?image=main.jpg" ALT="Main Page" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&cache=$cache">
<IMG SRC="rpm.cgi?image=package.jpg" ALT="Package Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=build&cache=$cache">
<IMG SRC="rpm.cgi?image=build.jpg" ALT="Build Detail" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=files&cache=$cache">
<IMG SRC="rpm.cgi?image=files.jpg" ALT="Files" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=deps&cache=$cache">
<IMG SRC="rpm.cgi?image=dependent.jpg" ALT="Dependency Information" BORDER=0></A>
<A HREF="rpm.cgi?rpm=$rpm&op=change&cache=$cache">
<IMG SRC="rpm.cgi?image=change.jpg" ALT="Change Log" BORDER=0></A>
<IMG SRC="rpm.cgi?image=help.jpg" ALT="Help" BORDER=0>
<BR><BR>
</CENTER>
</BODY>
</HTML>
END_HTML

}

#############################################################################

# Below are the help items associated witht the various operations that RPM.CGI can perform

sub help_dependents {

    print <<END_HELP;
<P>RPMs can provide capabilities that other RPMs may use. This may be a shared object (a .so file)
that can be used by other programs. It may be a particular program that other RPMs are known to
rely on. Finally it may be an abstract capability, for example, wu-ftpd provides a "ftpserver"
capability.</P>
<P>The Dependency Information page gives you details of the dependencies relevant to the current 
package. It shows you which capabilities that package provides, which it requires and which 
capabilities are now obsolete.</P>
<P>If a package requires a cabability the package that provides that capability is listed as a link 
next to it.</P>
END_HELP
}


sub help_file_checking {

    print <<END_HELP;
<P>The file checking page dispays information about the files that have been installed on the 
system that helps to highlight if there have been any malicous changes since install time.</P>
<P>There are eight tests carried out on each file and a green tick indicates that the tests 
have been successful. The tests are:</P>
<BLOCKQUOTE>
<P><B>S</B> - The <B>file size</B> of each file installed by the package is kept in the RPM database 
if any of the files have been modified by even a byte this check will let you know. It will 
probably be no problem if a configuration file has changed in size but for an executable this 
could indicate a serious problem</P>
<P><B>M</B> - The <B>Mode bits</B> indicate the permissions of a file and who has read, write and 
execute permissions. These are important to pay attention to because these could allow someone 
to have the permission of root user for the duration of a program if they are set in a certain 
way. This check will fail if the persmiisions have been changed from the ones set at install 
time.</P>
<P><B>5</B> - The <B>MD5 Checksum</B> is basically a 128 bit number that is created using the MD5 
algorithm and ensures that if even the smallest change to the file is made the checksum would 
be different. RPM created a MD5 checksum when every file is installed. This test will therefore 
fail if even the smallest change has been made to the file since install time.</P>
<P><B>D</B> - The <B>Device check</B> compares information on the Major and Minor device numbers that 
are in the RPM database and compares that too what is now on the system. This test will fail 
if the device numbers have been altered in anyway. This test is applicable to the special 
files found in the /dev directory.</P>
<P><B>L</B> - The <B>Link Check</B> is applicable to installed files that are actually links 
to other files. This test fails if the link has been changed to point to another file.</P>
<P><B>U</B> - The <B>User Owner</B> of each file is also recorded in the RPM database and this test 
will fail if the User Owner has been changed to a different user since install time.</P>
<P><B>G</B> - The <B>Group Owner</B> is similar to above but the test is carried out to 
see if the Group Owner has changed since install time.</P>
<P><B>T</B> - The <B>Modification Time</B> of each file is installed in the RPM database and this 
test will fail if the modification time of the file is different to that in the RPM database.</P>
</BLOCKQUOTE>
<P>For packages with many files the page may be split up into several pages. The size of the page
can be configured in the configuration file /etc/rpm_cgi/rpm_cgi.conf</P>
END_HELP
}


sub help_build_detail {

    print <<END_HELP;
The Build Detail page shows you information on when and who built the RPM that is installed 
on the system. It gives details of the time the build took place and the name of the 
machine it was built on. Also, in many cases the Linux distribution that the package 
appeared in will be listed and the vendor of the package.
END_HELP
}


sub help_package_information {

    print <<END_HELP;
The Package Information screen contains all the general information about the RPM package 
that is installed on the system. A description of the package tell us what the package does. 
Other information includes the time the package was installed on the system, the package group 
that the package is a member of and the Copyright type of the package.
END_HELP
}

sub help_change_log {

    print <<END_HELP;
The Change Log page includes the list of changes that have been recorded with the RPMs. 
These sometimes refer to the changes made to the RPM and sometimes refer to the changes 
made to the actual program depending on who made the RPM. This page can be helpful to 
show what has been added to the RPM in a latest release or who to find out who has been 
making the changes to the RPM.
END_HELP
}


#############################################################################

# this function takes a date as an epoch and prints the real date

sub print_date {

    local ($date) = @_;
    
    if($date >= 946684800) {
	# Year 2K fix ;)
	$year_prefix = "20";
    }
    else {
	$year_prefix = "19";
    }
    
    # Get the local time zone
    $timezn = `$date_prog +%Z`;
    chomp($timezn);

    # fill the array returned from the tm struct
    ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime $date;
    
    @days = qw{ Sun Mon Tue Wed Thu Fri Sat };
    @months = qw{ Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec };
    
    print "$days[$wday] $months[$mon] $mday $hour:$min:$sec $timezn $year_prefix$year";
}

sub image_print {

    local ($image_name) = @_;


    # Check the suffix for the file type
    $suffix = substr $image_name, -3;

    # Put the suffix into lowercase
    $suffix =~ tr/A-Z/a-z/;
    if($suffix eq "gif") {
	print "Content-type: image/gif \n\n";
    }
    elsif($suffix eq "jpg") {
	print "Content-type: image/jpeg \n\n";
    }
    else {
	die "Not sure of this image type: $suffix";
    }
    
    # Open the require image or die
    open(IMAGE, "$image_loc/$image_name") or die "Couldn't open image $image_name: $!";

    # print the image to stdout
    while($line = <IMAGE>) {
	print $line;
    }
    close(IMAGE);
    exit(0);
}




